//= require <dSS/util/Util>
Ext.namespace('dSS');

dSS.ScenesPanel = Ext.extend(Ext.Panel, {
    initComponent: function() {

        var contentPanel = {
            ref: 'contentPanel',
            region: 'center',
            layout: 'card',
            hideBorders: true,
            border: true,
            activeItem: 0,
            items: [
            {
                xtype: 'dssdevicepanel',
                featureConfig: dSS.config.FeatureConfig.SCENE
            }]
        };

        this.treePanel = new Ext.tree.TreePanel({
            useArrows: true,
            autoScroll: true,
            animate: true,
            enableDD: false,
            containerScroll: true,
            border: false,
            rootVisible: false,
            root: new Ext.tree.AsyncTreeNode(),
            listeners: {
                click: this.handleNodeClick.createDelegate(this)
            }
        });

        this.treePanel.getRootNode().expand();

        this.treeHolder = new Ext.Panel({
            title: dSS._("Scene", 2),
            region: 'west',
            layout: 'fit',
            split: true,
            width: 225,
            minSize: 175,
            maxSize: 400,
            tbar: this.buildTopToolbar(),
            items: [ this.treePanel ]
        });

        Ext.apply(this, {
            layout: 'border',
            border: false,
            bodyBorder: false,
            items: [this.treeHolder, contentPanel]
        });

        this.dirtyTree = false;
        this.updateTree();

        dSS.data.Loader.getZoneStore().addListener('datachanged',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getZoneStore().addListener('update',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getSceneStore().addListener('datachanged',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.data.Loader.getSceneStore().addListener('update',
                function(store) {
                    this.dirtyTree = true;
                    if (this.isVisible() === true) {
                        this.fireEvent('redrawtree');
                    }
                }, this);

        dSS.ScenesPanel.superclass.initComponent.apply(this, arguments);

        this.on('afterrender', function() {
                if (dSS.config.firstLoad === true) {
                    dSS.data.Loader.load();
                    dSS.config.firstLoad = false;
                }
            },
            this);
        this.on('activate', function() {
            if (this.activeNode !== undefined) {
                this.activeNode.setCls();
                this.activeNode = undefined;
            }
            this.handleTreeView();
        },
        this);

        this.on('redrawtree', this.handleTreeView.createDelegate(this), this,
                { buffer: 1000 });

        dSS.MessageBus.on('scenesmarkdirty',
                           this.markDirty.createDelegate(this), this);
        this.treePanel.on('contextmenu', function(node, event) {
            event.preventDefault();
            event.stopEvent();

            if (!node.isSelected()) {
                node.select();
                this.handleNodeClick(node);
            }

            if (!node.isLeaf()) {
                return;
            }

            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var menuItem = new Ext.menu.Item({
                text: dSS._("Call scene"),
                iconCls: 'callSceneIcon',
                handler: this.callScene.createDelegate(this, [node])
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            menuItem = new Ext.menu.Item({
                text: dSS._("Save scene"),
                iconCls: 'saveScene',
                handler: this.saveScene.createDelegate(this, [node])
            });
            this.contextMenu.add(menuItem);

            var path = node.getPath().split('/').slice(1);
            if (path.length === 4) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Rename scene"),
                    iconCls: 'editIcon',
                    handler: this.renameSelectedScene.createDelegate(this, [node])
                });
                this.contextMenu.add(menuItem);
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },
    markDirty: function() {
        this.dirtyTree = true;
        if (this.isVisible() === true) {
            this.fireEvent('redrawtree');
        }
    },

    handleTreeView: function() {
        if (this.dirtyTree === true) {
            this.updateTree();
            return;
        }

        var selModel = this.treePanel.getSelectionModel();
        var node = selModel.getSelectedNode();
        var root = this.treePanel.getRootNode();
        if (node === null) {
            if (root.hasChildNodes()) {
                root.firstChild.select();
                this.handleNodeClick(root.firstChild);
            } else {
              this.updateTree();
            }
        } else {
            this.handleNodeClick(node);
        }
    },

    getSceneName: function(zone, group, scene) {
        var sceneStore = dSS.data.Loader.getSceneStore();
        var rec = sceneStore.getById(zone + '-' + group + '-' + scene);
        if (rec === undefined) {
            return '';
        }

        var name = rec.get('name');
        if ((name === undefined) || (name.length === 0)) {
            return '';
        }

        return ' - ' + name;
    },

    generateSceneNode: function(zone, group, scene, icon) {
        var node = {};
        node.text = dSS._(dSS.config.sceneNames[scene]) +
                    this.getSceneName(zone, group, scene);
        node.id = zone + '-' + group + '-' + scene;
        node.leaf = true;
        node.iconCls = icon;
        return node;
    },

    // generates scenes 0-4 for a given zone/group
    generate0to4Scenes: function(zone, group, icon) {
        var addScenes =
        [
            dSS.config.scenes.T0_S0, dSS.config.scenes.T0_S1,
            dSS.config.scenes.T0_S2, dSS.config.scenes.T0_S3,
            dSS.config.scenes.T0_S4
        ];
        var scenes = [];

        var i;
        for (i = 0; i < addScenes.length; i++) {
            scenes.push(
                this.generateSceneNode(zone, group, addScenes[i], icon));
        }
        return scenes;
    },

    // for leafs the id is encoded as: zone id - group - scene number
    // when no group is desired, 0 is used
    generateSubNodes: function(zoneId, groups) {
        var retNodes = [];
        if (groups.LIGHT === true) {
            retNodes.push(
            {
                text: dSS._("Light scenes"),
                id: zoneId + '-' + dSS.config.groups.LIGHT + '-all',
                leaf: false,
                children: this.generate0to4Scenes(zoneId, 
                                                  dSS.config.groups.LIGHT,
                                                  'lightbulbOn')
            });
        }

        if (groups.SHADOW === true) {
            retNodes.push({
                text: dSS._("Shade scenes"),
                id: zoneId + '-' + dSS.config.groups.SHADOW + '-all',
                leaf: false,
                children: this.generate0to4Scenes(zoneId,
                                                  dSS.config.groups.SHADOW,
                                                  'shadesIcon')
            });
        }

        if (groups.ACCESS === true) {
            retNodes.push({
                text: dSS._("Access scenes"),
                id: zoneId + '-' + dSS.config.groups.ACCESS + '-all',
                leaf: false,
                children: this.generate0to4Scenes(zoneId,
                                                  dSS.config.groups.ACCESS,
                                                  'accessIcon')
            });
        }

        if (groups.SECURITY === true) {
            retNodes.push({
                text: dSS._("Security scenes"),
                id: zoneId + '-' + dSS.config.groups.SECURITY + '-all',
                leaf: false,
                children: this.generate0to4Scenes(zoneId,
                                                  dSS.config.groups.SECURITY,
                                                  'securityIcon')
            });
        }

        retNodes.push(
        {
            text:  dSS._("Activities"),
            id: zoneId + '-' + 'all-all',
            leaf: false,
            children:
            [
                {
                    text: dSS._("Standby"),
                    id: zoneId + '-all-' + dSS.config.scenes.STANDBY,
                    leaf: true,
                    iconCls: 'standByIcon'
                },
                {
                    text: dSS._("Deep off"),
                    id: zoneId + '-all-' + dSS.config.scenes.DEEP_OFF,
                    leaf: true,
                    iconCls: 'deepOffIcon'
                },
                {
                    text: dSS._("Sleeping"),
                    id: zoneId + '-all-' + dSS.config.scenes.SLEEPING,
                    leaf: true,
                    iconCls: 'sleepingIcon'
                },
                {
                    text: dSS._("Wake up"),
                    id: zoneId + '-all-' + dSS.config.scenes.WAKE_UP,
                    leaf: true,
                    iconCls: 'wakeupIcon'
                }
            ]
        });

        return retNodes;
    },

    genererateApartmentNodes: function() {
        return [
            {
                text:  dSS._("Activities"),
                id: '0-all-all',
                leaf: false,
                children:
                [
                    {
                        text: dSS._("Leave home"),
                        id: '0' + '-all-' + dSS.config.scenes.ABSENT,
                        leaf: true,
                        iconCls: 'leaveHomeIcon'
                    },
                    {
                        text: dSS._("Coming home"),
                        id: '0' + '-all-' + dSS.config.scenes.PRESENT,
                        leaf: true,
                        iconCls: 'comeHomeIcon'
                    },
                    {
                        text: dSS._("Door bell"),
                        id: '0' + '-all-' + dSS.config.scenes.SIG_BELL,
                        leaf: true,
                        iconCls: 'doorBellIcon'
                    },
                    {
                        text: dSS._("Panic"),
                        id: '0' + '-all-' + dSS.config.scenes.PANIC,
                        iconCls: 'panicIcon',
                        leaf: true
                    }
                ]
            }
        ]
    },

    updateTree: function() {
        var devStore = dSS.data.Loader.getDeviceStore();
        devStore.clearFilter();

        var selModel = this.treePanel.getSelectionModel();
        var selectedNode = selModel.getSelectedNode();
        var selectedPath = '';
        if (selectedNode !== null) {
            selectedPath = selectedNode.getPath();
        }

        var zStore = dSS.data.Loader.getZoneStore();
        var nodes = [];
        zStore.each(
        function(record) {
            if ((record.get('id') !== 0) && (devStore.find('zoneId', record.get('id')) >= 0)) {
                var deviceTypesInZone = {};
                var addZone = false;
                devStore.each(function(device) {
                    if (device.get('zoneId') === record.get('id')) {
                        if (device.get('isPresent') === false) {
                            return;
                        }

                        var fid = dSS.util.parseFunctionID(device.get('functionID'));
                        switch (fid[0]) {
                            case dSS.config.groups.LIGHT:
                               deviceTypesInZone.LIGHT = true;
                               addZone = true;
                               break;
                            case dSS.config.groups.SHADOW:
                                deviceTypesInZone.SHADOW = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.CLIMATE:
                                deviceTypesInZone.CLIMATE = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.AUDIO:
                                deviceTypesInZone.AUDIO = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.VIDEO:
                                deviceTypesInZone.VIDEO = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.SECURITY:
                                deviceTypesInZone.SECURITY = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.ACCESS:
                                deviceTypesInZone.ACCESS = true;
                               addZone = true;
                                break;
                            case dSS.config.groups.JOKER:
                                deviceTypesInZone.JOKER = true;
                               addZone = true;
                                break;
                            default:
                                break;
                        }
                    }
                });
                if (addZone === true) {
                    nodes.push({
                        text: record.get('name'),
                        id: record.get('id'),
                        leaf: false,
                        children: this.generateSubNodes(record.get('id'),
                                                        deviceTypesInZone)
                    });
                }
            }
            return true;
        },
        this);

        nodes = nodes.concat(this.genererateApartmentNodes());

        var root = new Ext.tree.AsyncTreeNode({
            children: nodes,
            id: 'root',
            leaf: false
        });
        this.treePanel.setRootNode(root);

        if (selectedPath.length > 0) {
            this.treePanel.selectPath(selectedPath, null, function(result, node) {
                if (result === false) {
                    if (root.firstChild !== null) {
                        root.firstChild.select();
                    }
                }
            });
        } else {
            if (root.firstChild !== null) {
                root.firstChild.select();
            }
        }

        if (selModel.getSelectedNode() !== null) {
            this.handleNodeClick(selModel.getSelectedNode());
        }

        if (this.activeNode !== undefined) {
            this.activeNode = this.treePanel.getNodeById(this.activeNode.id);
            if (this.activeNode !== undefined) {
                this.activeNode.setCls('active-scene-node');
            }
        }

        this.dirtyTree = false;
    },

    handleNodeClick: function(node) {
        if (!this.isVisible()) {
            return;
        }

        if (node === null) {
            return;
        }

        if (node.isLeaf() === true) {
            this.treeHolder.getTopToolbar().get('callSceneButton').enable();
            this.treeHolder.getTopToolbar().get('saveSceneButton').enable();
        } else {
            this.treeHolder.getTopToolbar().get('callSceneButton').disable();
            this.treeHolder.getTopToolbar().get('saveSceneButton').disable();
        }

        // we are on the zone node
        var path = node.getPath().split('/').slice(1);
        if (path.length === 4) {
            this.treeHolder.getTopToolbar().get('renameSceneButton').enable();
        } else {
            this.treeHolder.getTopToolbar().get('renameSceneButton').disable();
        }

        if (path.length < 2) {
            return;
        }

        var store = dSS.data.Loader.getDeviceStore();
        store.clearFilter();
        store.filterBy(function(record) {
            var zoneId = parseInt(path[1], 10);

            if (record.get('isPresent') === false) {
                return false;
            }

            if (zoneId === 0) {
                return true;
            }

            if (record.get('zoneId') === zoneId) {
                if (path.length === 2) {
                    return true;
                }

                var fid = dSS.util.parseFunctionID(record.get('functionID'));

                var nodeType = path[2].split('-');
                // all leaf node ids will have 3 components: zone-group-scene
                if (nodeType.length < 3) {
                    return false;
                }

                // if the group is set, we sort only by group
                if (nodeType[1] !== 'all') {
                    var group = parseInt(nodeType[1], 10);
                    if (!isNaN(group)) {
                        if (fid[0] === group) {
                            return true;
                        }

                        return false;
                    }
                } else {
                    return true;
                }

                // at this stage groups do not matter, scene filtering applies
                var scene = parseInt(nodeType[2], 10);

                if (isNaN(scene)) {
                    return false;
                }
            }
            return false;
        });
    },
    buildTopToolbar: function() {
        return [
            {
                iconCls: 'editIcon',
                handler: this.renameSelectedScene.createDelegate(this),
                tooltip: dSS._("Rename selected scene"),
                id: 'renameSceneButton',
                disabled: true,
                scope: this
            },
            {
                iconCls: 'saveScene',
                handler: this.saveSelectedScene.createDelegate(this),
                id: 'saveSceneButton',
                tooltip: dSS._("Save selected scene"),
                disabled: true,
                scope: this
            },
            '-',
            {
                iconCls: 'callSceneIcon',
                handler: this.callSelectedScene.createDelegate(this),
                id: 'callSceneButton',
                tooltip: dSS._("Call selected scene"),
                disabled: true,
                scope: this
            }
        ];
    },

    getZoneSceneGroup: function(id) {
        var ret = {
            "zoneId": null,
            "sceneNumber": null,
            "groupId": null
        };

        var ids = id.split('-');
        if (ids.length < 3) {
            return ret;
        }

        var zoneId = parseInt(ids[0], 10);
        if (isNaN(zoneId)) {
            return ret;
        }

        ret.zoneId = zoneId;

        var groupId = parseInt(ids[1], 10);
        if (isNaN(groupId)) {
            groupId = null;
        }

        ret.groupId = groupId;

        var sceneNumber = parseInt(ids[2], 10);
        if (isNaN(sceneNumber)) {
            sceneNumber = null;
        }

        ret.sceneNumber = sceneNumber;

        return ret;
    },

    getSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();
        if ((node === null) || (!node.isLeaf())) {
            throw dSS._("Can not identify scene: no scene selected");
        }

        var ids = this.getZoneSceneGroup(node.id);
        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            throw dSS._("Can not identify scene: invalid data");
        }

        return ids.sceneNumber;
    },

    callSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();
        if ((node === null) || (!node.isLeaf())) {
            Ext.Msg.alert(dSS._("Error"), dSS._("No scene selected, please select the scene that you would like to call"));
            return;
        }

        this.callScene(node);
    },

    callScene: function(node) {
        if ((node === null) || (!node.isLeaf())) {
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        if (this.activeNode !== undefined) {
            this.activeNode.setCls();
        }

        node.setCls('active-scene-node');
        this.activeNode = node;
        dSS.util.callSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);
    },
    saveSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();
        if ((node === null) || (!node.isLeaf())) {
            Ext.Msg.alert(dSS._("No scene selected"), dSS._("No scene selected, please select the scene that you would like to save"));
            return;
        }

        this.saveScene(node);
    },
    saveScene: function(node) {
        if ((node === null) || (!node.isLeaf())) {
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        if (ids.sceneNumber !== 0) {
            if ((this.activeNode !== undefined) && (node !== this.activeNode)) {
                this.warnSaveDifferentScene(node, this.activeNode);
                return;
            }
            dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);
        } else {
            this.warnSaveScene0(node);
        }
    },
    warnSaveDifferentScene: function(node, activeNode) {
        var ids = this.getZoneSceneGroup(node.id);
        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        var aIds = this.getZoneSceneGroup(activeNode.id);
        if ((aIds.zoneId === null) || (aIds.sceneNumber === null)) {
            return;
        }

        var calledName = dSS._(dSS.config.sceneNames[aIds.sceneNumber]) +
            this.getSceneName(aIds.zoneId, aIds.groupId, aIds.sceneNumber);

        var savingName = dSS._(dSS.config.sceneNames[ids.sceneNumber]) +
            this.getSceneName(ids.zoneId, ids.groupId, ids.sceneNumber);

        var title = dSS.gt.strargs(dSS._("Save scene %1%2%3"),
            [
                '&quot;', savingName, '&quot;'
            ]);

        var msg = dSS._("Called scene") + ': &quot;' + calledName +
                        '&quot;<br/>' +
                  dSS._("Saving scene") + ': &quot;' + savingName +
                        '&quot;<br/><br/>' +
                  dSS._("You are about to save a scene that is different " +
                        "from the called scene.") + '<br/>' +
                  dSS._("Are you sure that you want to overwrite the scene?");

        Ext.Msg.confirm(
            title,
            msg,
            function(btn) {
                if (btn === 'yes') {
                    dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber,
                                             ids.groupId);
                }
            },
            this
        );
    },
    warnSaveScene0: function(node) {
        var ids = this.getZoneSceneGroup(node.id);
        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        var savingName = dSS._(dSS.config.sceneNames[ids.sceneNumber]) +
            this.getSceneName(ids.zoneId, ids.groupId, ids.sceneNumber);

        var title = dSS.gt.strargs(dSS._("Save scene %1%2%3"),
            [
                '&quot;', savingName, '&quot;'
            ]);

        var msg = dSS._("You are about to save scene 0 which represents " +
                        "the off scene. Note that you might not be able to " +
                        "switch off all devices in case some devices in the " +
                        "selected room are turned on right now.") + '<br/>' +
                  dSS._("Are you sure you want to save the scene?");

        Ext.Msg.confirm(
            title,
            msg,
            function(btn) {
                if (btn === 'yes') {
                    dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber,
                                             ids.groupId);
                }
            },
            this
        );
    },
    renameSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();
        if ((node === null) || (!node.isLeaf())) {
            Ext.Msg.alert(dSS._("Error"), dSS._("No scene selected, please select the scene that you want to rename"));
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) ||
            (ids.groupId === null) || (ids.sceneNumber === null)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Could not determine scene from node"));
            return;
        }

        var sceneStore = dSS.data.Loader.getSceneStore();
        var recId = ids.zoneId + '-' + ids.groupId + '-' + ids.sceneNumber;

        var record = sceneStore.getById(recId);

        // this should never be the case, however if by any chance we missed
        // something I prefer having a messagebox here instead of some
        // random error
        if (record === undefined) {
            record = new sceneStore.recordType({
                                                    name : "",
                                                    scene: ids.sceneNumber,
                                                    group: ids.groupId,
                                                    zone: ids.zoneId
                                                }, recId);
            sceneStore.add([record]);
        }

        Ext.Msg.prompt(dSS._("Rename scene"), dSS._("New name for scene"),
                function(btn, text) {
                if (text !== record.get('name')) {
                    Ext.Ajax.request({
                        url: '/json/zone/sceneSetName',
                        disableCaching: true,
                        method: "GET",
                        params: {
                            id: ids.zoneId,
                            groupID: ids.groupId,
                            sceneNumber: ids.sceneNumber,
                            newName: text
                        },
                        success: function(result, request) {
                            try {
                                var jsonData = Ext.util.JSON.decode(result.responseText);
                                if (jsonData.ok) {
                                    var old = record.get('name');
                                    record.set('name', text);
                                    sceneStore.commitChanges();
                                    dSS.MessageBus.fireEvent('statusmessage',
                                        dSS._("Scene") + ' ' +
                                        dSS.util.getName(old) + ' ' +
                                        ' ' + dSS._("was renamed") + '.' +
                                        dSS._("New name") + ': ' + text);
                                } else {
                                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename scene"));
                                }
                            }
                            catch(err) {
                                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename scene") + ': ' + err);
                            }
                        },
                        failure: function(result, request) {
                            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not rename scene"));
                        }
                    });
                }
            },
            this, false, record.get('name'));
    }
});

Ext.reg('dssscenespanel', dSS.ScenesPanel);

